@@ -1,19 +1,11 @@
 ##	common/config.py
 ##
-## Contributors for this file:
-##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Nikos Kouremenos <nkour@jabber.org>
-##	- Dimitur Kirov <dkirov@gmail.com>
-##	- Travis Shirk <travis@pobox.com>
-##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
+## Copyright (C) 2004-2005 Vincent Hanquez <tab@snarc.org>
+## Copyright (C) 2005 Dimitur Kirov <dkirov@gmail.com>
+## Copyright (C) 2005 Travis Shirk <travis@pobox.com>
+## Copyright (C) 2005 Norman Rasmussen <norman@rasmussen.co.za>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
@@ -31,20 +23,27 @@ import copy
 import i18n
 _ = i18n._
 
-OPT_TYPE = 0
-OPT_VAL = 1
-OPT_DESC = 2
+
+(
+OPT_TYPE,
+OPT_VAL,
+OPT_DESC,
+# If OPT_RESTART is True - we need restart to use our changed option
+# OPT_DESC also should be there
+OPT_RESTART,
+) = range(4)
 
 opt_int = [ 'integer', 0 ]
 opt_str = [ 'string', 0 ]
 opt_bool = [ 'boolean', 0 ]
 opt_color = [ 'color', '^(#[0-9a-fA-F]{6})|()$' ]
+opt_one_window_types = ['never', 'always', 'peracct', 'pertype']
 
 class Config:
 
 	__options = {
-		# name: [ type, value ]
-		'verbose': [ opt_bool, False ],
+		# name: [ type, default_value, help_string ]
+		'verbose': [ opt_bool, False, '', True ],
 		'alwaysauth': [ opt_bool, False ],
 		'autopopup': [ opt_bool, False ],
 		'notify_on_signin': [ opt_bool, True ],
@@ -53,7 +52,7 @@ class Config:
 		'autopopupaway': [ opt_bool, False ],
 		'use_notif_daemon': [ opt_bool, True , _('Use DBus and Notification-Daemon to show notifications') ],
 		'ignore_unknown_contacts': [ opt_bool, False ],
-		'showoffline': [ opt_bool, False ],
+		'showoffline': [ opt_bool, False, '', True ],
 		'autoaway': [ opt_bool, True ],
 		'autoawaytime': [ opt_int, 5, _('Time in minutes, after which your status changes to away.') ],
 		'autoaway_message': [ opt_str, _('Away as a result of being idle') ],
@@ -69,24 +68,26 @@ class Config:
 		'last_status_msg_dnd': [ opt_str, '' ],
 		'last_status_msg_invisible': [ opt_str, '' ],
 		'last_status_msg_offline': [ opt_str, '' ],
-		'trayicon': [ opt_bool, True ],
-		'iconset': [ opt_str, 'dcraven' ],
-		'use_transports_iconsets': [ opt_bool, True ],
-		'inmsgcolor': [ opt_color, '#a34526' ],
-		'outmsgcolor': [ opt_color, '#164e6f' ],
-		'statusmsgcolor': [ opt_color, '#1eaa1e' ],
-		'markedmsgcolor': [ opt_color, '#ff8080' ],
-		'urlmsgcolor': [ opt_color, '#0000ff' ],
-		'collapsed_rows': [ opt_str, '' ],
-		'roster_theme': [ opt_str, 'green' ],
+		'trayicon': [ opt_bool, True, '', True ],
+		'iconset': [ opt_str, 'dcraven', '', True ],
+		'use_transports_iconsets': [ opt_bool, True, '', True ],
+		'inmsgcolor': [ opt_color, '#a34526', '', True ],
+		'outmsgcolor': [ opt_color, '#164e6f', '', True ],
+		'statusmsgcolor': [ opt_color, '#1eaa1e', '', True ],
+		'markedmsgcolor': [ opt_color, '#ff8080', '', True ],
+		'urlmsgcolor': [ opt_color, '#0000ff', '', True ],
+		'collapsed_rows': [ opt_str, '', '', True ],
+		'roster_theme': [ opt_str, 'gtk+', '', True ],
 		'saveposition': [ opt_bool, True ],
-		'mergeaccounts': [ opt_bool, False ],
-		'sort_by_show': [ opt_bool, True ],
-		'usetabbedchat': [ opt_bool, True ],
-		'use_speller': [ opt_bool, False ],
+		'mergeaccounts': [ opt_bool, False, '', True ],
+		'sort_by_show': [ opt_bool, True, '', True ],
+		'use_speller': [ opt_bool, False, ],
 		'print_time': [ opt_str, 'always' ],
-		'useemoticons': [ opt_bool, True ],
-		'show_ascii_formatting_chars': [ opt_bool, False , _('If True, do not remove */_ . So *abc* will be bold but with * * not removed.')],
+		'emoticons_theme': [opt_str, 'static', '', True ],
+		'ascii_formatting': [ opt_bool, True,
+			_('Treat * / _ pairs as possible formatting characters.'), True],
+		'show_ascii_formatting_chars': [ opt_bool, False , _('If True, do not '
+			'remove */_ . So *abc* will be bold but with * * not removed.')],
 		'sounds_on': [ opt_bool, True ],
 		# 'aplay', 'play', 'esdplay', 'artsplay' detected first time only
 		'soundplayer': [ opt_str, '' ],
@@ -94,21 +95,25 @@ class Config:
 		'custombrowser': [ opt_str, 'firefox' ],
 		'custommailapp': [ opt_str, 'mozilla-thunderbird -compose' ],
 		'custom_file_manager': [ opt_str, 'xffm' ],
-		'gc-x-position': [opt_int, 0],
-		'gc-y-position': [opt_int, 0],
-		'gc-width': [opt_int, 675],
-		'gc-height': [opt_int, 400],
 		'gc-hpaned-position': [opt_int, 540],
 		'gc_refer_to_nick_char': [opt_str, ','],
 		'gc_proposed_nick_char': [opt_str, '_'],
-		'chat-x-position': [opt_int, 0],
-		'chat-y-position': [opt_int, 0],
-		'chat-width': [opt_int, 480],
-		'chat-height': [opt_int, 440],
-		'single_msg-x-position': [opt_int, 0],
-		'single_msg-y-position': [opt_int, 0],
-		'single_msg-width': [opt_int, 400],
-		'single_msg-height': [opt_int, 280],
+		'msgwin-x-position': [opt_int, -1], # Default is to let the window manager decide
+		'msgwin-y-position': [opt_int, -1], # Default is to let the window manager decide
+		'msgwin-width': [opt_int, 480],
+		'msgwin-height': [opt_int, 440],
+		'chat-msgwin-x-position': [opt_int, -1], # Default is to let the window manager decide
+		'chat-msgwin-y-position': [opt_int, -1], # Default is to let the window manager decide
+		'chat-msgwin-width': [opt_int, 480],
+		'chat-msgwin-height': [opt_int, 440],
+		'gc-msgwin-x-position': [opt_int, -1], # Default is to let the window manager decide
+		'gc-msgwin-y-position': [opt_int, -1], # Default is to let the window manager decide
+		'gc-msgwin-width': [opt_int, 480],
+		'gc-msgwin-height': [opt_int, 440],
+		'single-msg-x-position': [opt_int, 0],
+		'single-msg-y-position': [opt_int, 0],
+		'single-msg-width': [opt_int, 400],
+		'single-msg-height': [opt_int, 280],
 		'roster_x-position': [ opt_int, 0 ],
 		'roster_y-position': [ opt_int, 0 ],
 		'roster_width': [ opt_int, 150 ],
@@ -120,7 +125,8 @@ class Config:
 		'before_nickname': [ opt_str, '' ],
 		'after_nickname': [ opt_str, ':' ],
 		'send_os_info': [ opt_bool, True ],
-		'usegpg': [ opt_bool, False ],
+		'notify_on_new_gmail_email': [ opt_bool, True ],
+		'usegpg': [ opt_bool, False, '', True ],
 		'use_gpg_agent': [ opt_bool, False ],
 		'change_roster_title': [ opt_bool, True, _('Add * and [n] in roster title?')],
 		'restore_lines': [opt_int, 4, _('How many lines to remember from previous conversation when a chat tab/window is reopened.')],
@@ -128,20 +134,20 @@ class Config:
 		'send_on_ctrl_enter': [opt_bool, False, _('Send message on Ctrl+Enter and with Enter make new line (Mirabilis ICQ Client default behaviour).')],
 		'show_roster_on_startup': [opt_bool, True],
 		'key_up_lines': [opt_int, 25, _('How many lines to store for Ctrl+KeyUP.')],
-		'version': [ opt_str, '0.9' ], # which version created the config
-		'always_compact_view_chat': [opt_bool, False, _('Use compact view when you open a chat window')],
-		'always_compact_view_gc': [opt_bool, False, _('Use compact view when you open a group chat window')],
+		'version': [ opt_str, '0.10' ], # which version created the config
 		'search_engine': [opt_str, 'http://www.google.com/search?&q=%s&sourceid=gajim'],
 		'dictionary_url': [opt_str, 'WIKTIONARY', _("Either custom url with %s in it where %s is the word/phrase or 'WIKTIONARY' which means use wiktionary.")],
 		'always_english_wikipedia': [opt_bool, False],
 		'always_english_wiktionary': [opt_bool, True],
-		'remote_control': [opt_bool, True, _('If checked, Gajim can be controlled remotely using gajim-remote.')],
+		'remote_control': [opt_bool, True, _('If checked, Gajim can be controlled remotely using gajim-remote.'), True],
 		'chat_state_notifications': [opt_str, 'all'], # 'all', 'composing_only', 'disabled'
-		'autodetect_browser_mailer': [opt_bool, True],
+		'autodetect_browser_mailer': [opt_bool, False, '', True],
 		'print_ichat_every_foo_minutes': [opt_int, 5],
 		'confirm_close_muc': [opt_bool, True, _('Ask before closing a group chat tab/window.')],
-		'notify_on_file_complete': [opt_bool, True], # notif. on file complete
-		'file_transfers_port': [opt_int, 28011],  # port, used for file transfers
+		'confirm_close_muc_rooms': [opt_str, '', _('Always ask before closing group chat tab/window in this space separated list of room jids.')],
+		'noconfirm_close_muc_rooms': [opt_str, '', _('Never ask before closing group chat tab/window in this space separated list of room jids.')],
+		'notify_on_file_complete': [opt_bool, True],
+		'file_transfers_port': [opt_int, 28011],
 		'ft_override_host_to_send': [opt_str, '', _('Overrides the host we send for File Transfer in case of address translation/port forwarding.')], 
 		'conversation_font': [opt_str, ''],
 		'use_kib_mib': [opt_bool, False, _('IEC standard says KiB = 1024 bytes, KB = 1000 bytes.')],
@@ -152,49 +158,70 @@ class Config:
 		'last_emoticons_dir': [opt_str, ''],
 		'last_sounds_dir': [opt_str, ''],
 		'tabs_position': [opt_str, 'top'],
-		'tabs_always_visible': [opt_bool, False, _('Show tab when only one conversation?')],
+		'tabs_always_visible': [opt_bool, False, _('Show tab when only one conversation?'), '', True],
 		'tabs_border': [opt_bool, False, _('Show tab border if one conversation?')],
 		'tabs_close_button': [opt_bool, True, _('Show close button in tab?')],
 		'chat_avatar_width': [opt_int, 52],
 		'chat_avatar_height': [opt_int, 52],
 		'roster_avatar_width': [opt_int, 32],
 		'roster_avatar_height': [opt_int, 32],
+		'tooltip_avatar_width': [opt_int, 125],
+		'tooltip_avatar_height': [opt_int, 125],
+		'vcard_avatar_width': [opt_int, 200],
+		'vcard_avatar_height': [opt_int, 200],
+		'notification_position_x': [opt_int, -1],
+		'notification_position_y': [opt_int, -1],
+		'notification_avatar_width': [opt_int, 48],
+		'notification_avatar_height': [opt_int, 48],
 		'muc_highlight_words': [opt_str, '', _('A semicolon-separated list of words that will be highlighted in multi-user chat.')],
 		'quit_on_roster_x_button': [opt_bool, False, _('If True, quits Gajim when X button of Window Manager is clicked. This setting is taken into account only if trayicon is used.')],
 		'set_xmpp://_handler_everytime': [opt_bool, False, _('If True, Gajim registers for xmpp:// on each startup.')],
 		'show_unread_tab_icon': [opt_bool, False, _('If True, Gajim will display an icon on each tab containing unread messages. Depending on the theme, this icon may be animated.')],
-		'show_status_msgs_in_roster': [opt_bool, True, _('If True, Gajim will display the status message, if not empty, for every contact under the contact name in roster window')],
-		'show_avatars_in_roster': [opt_bool, True],
+		'show_status_msgs_in_roster': [opt_bool, True, _('If True, Gajim will display the status message, if not empty, for every contact under the contact name in roster window'), True],
+		'show_avatars_in_roster': [opt_bool, True, '', True],
 		'ask_avatars_on_startup': [opt_bool, True, _('If True, Gajim will ask for avatar each contact that did not have an avatar last time or has one cached that is too old.')],
-		#FIXME: remvoe you and make it Gajim will not; and/or his or *her* status messages
+		#FIXME: remove you and make it Gajim will not; and/or his or *her* status messages
 		'print_status_in_chats': [opt_bool, True, _('If False, you will no longer see status line in chats when a contact changes his or her status and/or his status message.')],
 		'log_contact_status_changes': [opt_bool, False],
 		'restored_messages_color': [opt_str, 'grey'],
 		'hide_avatar_of_transport': [opt_bool, False],
+		'roster_window_skip_taskbar': [opt_bool, False],
+		'use_urgency_hint': [opt_bool, True, _('If True and installed GTK+ and PyGTK versions are at least 2.8, make the window flash (the default behaviour in most Window Managers) when holding pending events.')],
+		'notification_timeout': [opt_int, 5],
+		'send_sha_in_gc_presence': [opt_bool, True, _('Jabberd1.4 does not like sha info when one join a password protected room. Turn this option to False to stop sending sha info in groupchat presences')],
+		'one_message_window': [opt_str, 'always',
+			_('Controls the window where new messages are placed.\n\'always\' - All messages are sent to a single window.\n\'never\' - All messages get their own window.\n\'peracct\' - Messages for each account are sent to a specific window.\n\'pertype\' - Each message type (e.g., chats vs. groupchats) are sent to a specific window. Note, changing this option requires restarting Gajim before the changes will take effect')],
+		'show_avatar_in_chat': [opt_bool, True, _('If False, you will no longer see the avatar in the chat window')],
+		'escape_key_closes': [opt_bool, True, _('If True, pressing the escape key closes a tab/window')],
+		'always_hide_groupchat_buttons': [opt_bool, False, _('Hides the buttons in group chat window')],
+		'always_hide_chat_buttons': [opt_bool, False, _('Hides the buttons in two persons chat window')],
+		'hide_groupchat_banner': [opt_bool, False, _('Hides the banner in a group chat window')],
+		'hide_chat_banner': [opt_bool, False, _('Hides the banner in two persons chat window')],
+		'hide_groupchat_occupants_list': [opt_bool, False, _('Hides the room occupants list in groupchat window')],
 	}
 
 	__options_per_key = {
 		'accounts': ({
-			'name': [ opt_str, '' ],
-			'hostname': [ opt_str, '' ],
+			'name': [ opt_str, '', '', True ],
+			'hostname': [ opt_str, '', '', True ],
 			'savepass': [ opt_bool, False ],
 			'password': [ opt_str, '' ],
-			'resource': [ opt_str, 'gajim' ],
-			'priority': [ opt_int, 5 ],
-			'autoconnect': [ opt_bool, False ],
+			'resource': [ opt_str, 'gajim', '', True ],
+			'priority': [ opt_int, 5, '', True ],
+			'autoconnect': [ opt_bool, False, '', True ],
 			'autoreconnect': [ opt_bool, True ],
 			'active': [ opt_bool, True],
-			'proxy': [ opt_str, '' ],
-			'keyid': [ opt_str, '' ],
-			'keyname': [ opt_str, '' ],
-			'usessl': [ opt_bool, False ],
-			'use_srv': [ opt_bool, True ],
-			'use_custom_host': [ opt_bool, False ],
-			'custom_port': [ opt_int, 5222 ],
-			'custom_host': [ opt_str, '' ],
-			'savegpgpass': [ opt_bool, False ],
+			'proxy': [ opt_str, '', '', True ],
+			'keyid': [ opt_str, '', '', True ],
+			'keyname': [ opt_str, '', '', True ],
+			'usessl': [ opt_bool, False, '', True ],
+			'use_srv': [ opt_bool, True, '', True ],
+			'use_custom_host': [ opt_bool, False, '', True ],
+			'custom_port': [ opt_int, 5222, '', True ],
+			'custom_host': [ opt_str, '', '', True ],
+			'savegpgpass': [ opt_bool, False, '', True ],
 			'gpgpassword': [ opt_str, '' ],
-			'sync_with_global_status': [ opt_bool, False ],
+			'sync_with_global_status': [ opt_bool, False, ],
 			'no_log_for': [ opt_str, '' ],
 			'attached_gpg_keys': [ opt_str, '' ],
 			'keep_alives_enabled': [ opt_bool, True],
@@ -202,18 +229,20 @@ class Config:
 			'keep_alive_every_foo_secs': [ opt_int, 55 ],
 			# try for 2 minutes before giving up (aka. timeout after those seconds)
 			'try_connecting_for_foo_secs': [ opt_int, 60 ],
-			'max_stanza_per_sec': [ opt_int, 5],
 			'http_auth': [opt_str, 'ask'], # yes, no, ask
+			'dont_ack_subscription': [opt_bool, False, _('Jabberd2 workaround')],
 			# proxy65 for FT
 			'file_transfer_proxies': [opt_str, 
-			'proxy.jabber.org, proxy65.jabber.autocom.pl, proxy.jabber.cd.chalmers.se, proxy.netlab.cz, proxy65.jabber.ccc.de, proxy65.unstable.nl'] 
+			'proxy.jabber.org, proxy.netlab.cz, transfer.jabber.freenet.de, proxy.jabber.cd.chalmers.se'],
+			'use_ft_proxies': [opt_bool, True, _('If checked, Gajim will use your IP and proxies defined in file_transfer_proxies option for file transfer.'), True],
+			'msgwin-x-position': [opt_int, -1], # Default is to let the wm decide
+			'msgwin-y-position': [opt_int, -1], # Default is to let the wm decide
+			'msgwin-width': [opt_int, 480],
+			'msgwin-height': [opt_int, 440],
 		}, {}),
 		'statusmsg': ({
 			'message': [ opt_str, '' ],
 		}, {}),
-		'emoticons': ({
-			'path': [ opt_str, '' ],
-		}, {}),
 		'soundevents': ({
 			'enabled': [ opt_bool, True ],
 			'path': [ opt_str, '' ],
@@ -225,27 +254,24 @@ class Config:
 			'user': [ opt_str, '' ],
 			'pass': [ opt_str, '' ],
 		}, {}),
-		'ft_proxies65_cache': ({
-			'host': [ opt_str, ''],
-			'port': [ opt_str, '7777'],
-			'jid': [ opt_str, ''],
-		}, {}),
 		'themes': ({
-			'accounttextcolor': [ opt_color, 'black' ],
-			'accountbgcolor': [ opt_color, 'white' ],
-			'accountfont': [ opt_str, '' ],
-			'accountfontattrs': [ opt_str, 'B' ],
-			'grouptextcolor': [ opt_color, 'black' ],
-			'groupbgcolor': [ opt_color, 'white' ],
-			'groupfont': [ opt_str, '' ],
-			'groupfontattrs': [ opt_str, 'I' ],
-			'contacttextcolor': [ opt_color, 'black' ],
-			'contactbgcolor': [ opt_color, 'white' ],
-			'contactfont': [ opt_str, '' ],
-			'contactfontattrs': [ opt_str, '' ],
-			'bannertextcolor': [ opt_color, 'black' ],
-			'bannerbgcolor': [ opt_color, '' ],
-
+			'accounttextcolor': [ opt_color, 'black', '', True ],
+			'accountbgcolor': [ opt_color, 'white', '', True ],
+			'accountfont': [ opt_str, '', '', True ],
+			'accountfontattrs': [ opt_str, 'B', '', True ],
+			'grouptextcolor': [ opt_color, 'black', '', True ],
+			'groupbgcolor': [ opt_color, 'white', '', True ],
+			'groupfont': [ opt_str, '', '', True ],
+			'groupfontattrs': [ opt_str, 'I', '', True ],
+			'contacttextcolor': [ opt_color, 'black', '', True ],
+			'contactbgcolor': [ opt_color, 'white', '', True ],
+			'contactfont': [ opt_str, '', '', True ],
+			'contactfontattrs': [ opt_str, '', '', True ],
+			'bannertextcolor': [ opt_color, 'black', '', True ],
+			'bannerbgcolor': [ opt_color, '', '', True ],
+			'bannerfont': [ opt_str, '', '', True ],
+			'bannerfontattrs': [ opt_str, 'B', '', True ],
+				
 			# http://www.pitt.edu/~nisg/cis/web/cgi/rgb.html
 			# FIXME: not black but the default color from gtk+ theme
 			'state_active_color': [ opt_color, 'black' ],
@@ -255,75 +281,12 @@ class Config:
 			'state_gone_color': [ opt_color, 'grey' ],
 
 			# MUC chat states
-			'state_muc_msg': [ opt_color, 'mediumblue' ],
-			'state_muc_directed_msg': [ opt_color, 'red2' ],
+			'state_muc_msg_color': [ opt_color, 'mediumblue' ],
+			'state_muc_directed_msg_color': [ opt_color, 'red2' ],
+		}, {}),
+		'contacts': ({
+			'gpg_enabled': [ opt_bool, True ],
 		}, {}),
-	}
-
-	emoticons_default = {
-		':-)': '../data/emoticons/smile.png',
-		'(@)': '../data/emoticons/pussy.png',
-		'8)': '../data/emoticons/coolglasses.png',
-		':(': '../data/emoticons/unhappy.png',
-		':)': '../data/emoticons/smile.png',
-		':/': '../data/emoticons/frowning.png',
-		'(})': '../data/emoticons/hugleft.png',
-		':$': '../data/emoticons/blush.png',
-		'(Y)': '../data/emoticons/yes.png',
-		':-@': '../data/emoticons/angry.png',
-		':-D': '../data/emoticons/biggrin.png',
-		'(U)': '../data/emoticons/brheart.png',
-		'(F)': '../data/emoticons/flower.png',
-		':-[': '../data/emoticons/bat.png',
-		':>': '../data/emoticons/biggrin.png',
-		'(T)': '../data/emoticons/phone.png',
-		':-S': '../data/emoticons/frowning.png',
-		':-P': '../data/emoticons/tongue.png',
-		'(H)': '../data/emoticons/coolglasses.png',
-		'(D)': '../data/emoticons/drink.png',
-		':-O': '../data/emoticons/oh.png',
-		'(C)': '../data/emoticons/coffee.png',
-		'({)': '../data/emoticons/hugright.png',
-		'(*)': '../data/emoticons/star.png',
-		'B-)': '../data/emoticons/coolglasses.png',
-		'(Z)': '../data/emoticons/boy.png',
-		'(E)': '../data/emoticons/mail.png',
-		'(N)': '../data/emoticons/no.png',
-		'(P)': '../data/emoticons/photo.png',
-		'(K)': '../data/emoticons/kiss.png',
-		':-*': '../data/emoticons/kiss.png',
-		':*': '../data/emoticons/kiss.png',
-		'(R)': '../data/emoticons/rainbow.png',
-		':-|': '../data/emoticons/stare.png',
-		';-)': '../data/emoticons/wink.png',
-		';-(': '../data/emoticons/cry.png',
-		'(6)': '../data/emoticons/devil.png',
-		'>:)': '../data/emoticons/devil.png',
-		'>:-)': '../data/emoticons/devil.png',
-		'(L)': '../data/emoticons/heart.png',
-		'<3': '../data/emoticons/heart.png',
-		'(W)': '../data/emoticons/brflower.png',
-		':|': '../data/emoticons/stare.png',
-		':O': '../data/emoticons/oh.png',
-		';)': '../data/emoticons/wink.png',
-		';(': '../data/emoticons/cry.png',
-		':S': '../data/emoticons/frowning.png',
-		';\'-(': '../data/emoticons/cry.png',
-		':-(': '../data/emoticons/unhappy.png',
-		'8-)': '../data/emoticons/coolglasses.png',
-		'(B)': '../data/emoticons/beer.png',
-		':D': '../data/emoticons/biggrin.png',
-		'(8)': '../data/emoticons/music.png',
-		':@': '../data/emoticons/angry.png',
-		'B)': '../data/emoticons/coolglasses.png',
-		':-$': '../data/emoticons/blush.png',
-		':\'(': '../data/emoticons/cry.png',
-		':->': '../data/emoticons/biggrin.png',
-		':[': '../data/emoticons/bat.png',
-		'(I)': '../data/emoticons/lamp.png',
-		':P': '../data/emoticons/tongue.png',
-		'(%)': '../data/emoticons/cuffs.png',
-		'(S)': '../data/emoticons/moon.png',
 	}
 
 	statusmsg_default = {
@@ -348,33 +311,25 @@ class Config:
 
 	themes_default = {
 		# sorted alphanum
-		_('green'): [ '#ffffff', '#94aa8c', '', 'B', '#0000ff', '#eff3e7',
-					'', 'I', '#000000', '#ffffff', '', '', '#ffffff',
-					'#94aa8c' ],
-		
-		_('grocery'): [ '#ffffff', '#6bbe18', '', 'B', '#12125a', '#ceefad',
-					'', 'I', '#000000', '#efb26b', '', '', '#ffffff',
-					'#108abd' ],
-		
-		_('human'): [ '#ffffff', '#996442', '', 'B', '#ab5920', '#e3ca94',
-					'', 'I', '#000000', '#ffffff', '', '', '#ffffff',
-					'#996442' ],
-					
-		_('marine'): [ '#ffffff', '#918caa', '', 'B', '#0000ff', '#e9e7f3',
-					'', 'I', '#000000', '#ffffff', '', '', '#ffffff',
-					'#918caa' ],
-		
-		_('plain'): [ '', '', '', 'B', '', '','', 'I', '', '', '', '', '','' ],
-		
-	}
-	
-	ft_proxies65_default = {
-		'proxy.jabber.org': [ '208.245.212.98', '7777', 'proxy.jabber.org' ],
-		'proxy65.jabber.autocom.pl': ['213.134.161.52', '7777', 'proxy65.jabber.autocom.pl'],
-		'proxy.jabber.cd.chalmers.se': ['129.16.79.37', '7777', 'proxy.jabber.cd.chalmers.se'],
-		'proxy.netlab.cz': ['82.119.241.3', '7777', 'proxy.netlab.cz'],
-		'proxy65.jabber.ccc.de': ['217.10.10.196', '7777', 'proxy65.jabber.ccc.de'],
-		'proxy65.unstable.nl': ['84.107.143.192', '7777', 'proxy65.unstable.nl'],
+		'gtk+': [ '', '', '', 'B', '', '','', 'I', '', '', '', '', '','', '',
+			'B' ],
+
+		_('green'): [ '', '#94aa8c', '', 'B', '#0000ff', '#eff3e7',
+					'', 'I', '#000000', '', '', '', '',
+					'#94aa8c', '', 'B' ],
+
+		_('grocery'): [ '', '#6bbe18', '', 'B', '#12125a', '#ceefad',
+					'', 'I', '#000000', '#efb26b', '', '', '',
+					'#108abd', '', 'B' ],
+
+		_('human'): [ '', '#996442', '', 'B', '#ab5920', '#e3ca94',
+					'', 'I', '#000000', '', '', '', '',
+					'#996442', '', 'B' ],
+
+		_('marine'): [ '', '#918caa', '', 'B', '', '#e9e7f3',
+					'', 'I', '#000000', '', '', '', '',
+					'#918caa', '', 'B' ],
+
 	}
 
 	def foreach(self, cb, data = None):
@@ -452,6 +407,12 @@ class Config:
 		if len(self.__options[optname]) > OPT_DESC:
 			return self.__options[optname][OPT_DESC]
 
+	def get_restart(self, optname):
+		if not self.__options.has_key(optname):
+			return None
+		if len(self.__options[optname]) > OPT_RESTART:
+			return self.__options[optname][OPT_RESTART]
+
 	def add_per(self, typename, name): # per_group_of_option
 		if not self.__options_per_key.has_key(typename):
 #			raise RuntimeError, 'option %s does not exist' % typename
@@ -509,6 +470,40 @@ class Config:
 			return None
 		return obj[subname][OPT_VAL]
 
+	def get_desc_per(self, optname, key = None, subname = None):
+		if not self.__options_per_key.has_key(optname):
+			return None
+		dict = self.__options_per_key[optname][1]
+		if not key:
+			return None
+		if not dict.has_key(key):
+			return None
+		obj = dict[key]
+		if not subname:
+			return None
+		if not obj.has_key(subname):
+			return None
+		if len(obj[subname]) > OPT_DESC:
+			return obj[subname][OPT_DESC]
+		return None
+
+	def get_restart_per(self, optname, key = None, subname = None):
+		if not self.__options_per_key.has_key(optname):
+			return False
+		dict = self.__options_per_key[optname][1]
+		if not key:
+			return False
+		if not dict.has_key(key):
+			return False
+		obj = dict[key]
+		if not subname:
+			return False
+		if not obj.has_key(subname):
+			return False
+		if len(obj[subname]) > OPT_RESTART:
+			return obj[subname][OPT_RESTART]
+		return False
+
 	def __init__(self):
 		#init default values
 		for event in self.soundevents_default:
@@ -517,11 +512,3 @@ class Config:
 			self.set_per('soundevents', event, 'enabled', default[0])
 			self.set_per('soundevents', event, 'path', default[1])
 
-		# set initial cache values for proxie65 hosts
-		for proxy in self.ft_proxies65_default:
-			default = self.ft_proxies65_default[proxy]
-			self.add_per('ft_proxies65_cache', proxy)
-			self.set_per('ft_proxies65_cache', proxy, 'host', default[0])
-			self.set_per('ft_proxies65_cache', proxy, 'port', default[1])
-			self.set_per('ft_proxies65_cache', proxy, 'jid', default[2])
-		return
