@@ -1,17 +1,10 @@
 ##	common/helpers.py
 ##
-## Contributors for this file:
-##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Nikos Kouremenos <kourem@gmail.com>
-##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
+## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
+## Copyright (C) 2005
 ##                    Dimitur Kirov <dkirov@gmail.com>
 ##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
@@ -27,23 +20,27 @@ import sre
 import os
 import urllib
 import errno
+import select
 import sys
-import stat
-from pysqlite2 import dbapi2 as sqlite
+import sha
+from encodings.punycode import punycode_encode
 
 import gajim
-import logger
 import i18n
 from xmpp_stringprep import nodeprep, resourceprep, nameprep
 
 try:
 	import winsound # windows-only built-in module for playing wav
+	import win32api
+	import win32con
 except:
 	pass
 
 _ = i18n._
 Q_ = i18n.Q_
 
+special_groups = (_('Transports'), _('Not in Roster'), _('Observers'))
+
 class InvalidFormat(Exception):
 	pass
 
@@ -58,9 +55,9 @@ def parse_jid(jidstring):
 
 	# Search for delimiters
 	user_sep = jidstring.find('@')
-	res_sep  = jidstring.find('/')
+	res_sep = jidstring.find('/')
 
-	if user_sep == -1:		
+	if user_sep == -1:
 		if res_sep == -1:
 			# host
 			server = jidstring
@@ -159,7 +156,7 @@ def convert_bytes(string):
 				bytes = round(bytes/align, 1)
 				if use_kib_mib:
 					#GiB means gibibyte
-					suffix = _('%s GiB') 
+					suffix = _('%s GiB')
 				else:
 					#GB means gigabyte
 					suffix = _('%s GB')
@@ -178,7 +175,7 @@ def convert_bytes(string):
 				#KB means kilo bytes
 				suffix = _('%s KB')
 	else:
-		#B means bytes 
+		#B means bytes
 		suffix = _('%s B')
 	return suffix % unicode(bytes)
 
@@ -224,14 +221,14 @@ def get_uf_show(show, use_mnemonic = False):
 			uf_show = _('_Invisible')
 		else:
 			uf_show = _('Invisible')
-	elif show == 'not in the roster':
-		uf_show = _('Not in the roster')
+	elif show == 'not in roster':
+		uf_show = _('Not in Roster')
 	elif show == 'requested':
 		uf_show = Q_('?contact has status:Unknown')
 	else:
 		uf_show = Q_('?contact has status:Has errors')
 	return unicode(uf_show)
-	
+
 def get_uf_sub(sub):
 	if sub == 'none':
 		uf_sub = Q_('?Subscription we already have:None')
@@ -243,9 +240,9 @@ def get_uf_sub(sub):
 		uf_sub = _('Both')
 	else:
 		uf_sub = sub
-	
+
 	return unicode(uf_sub)
-	
+
 def get_uf_ask(ask):
 	if ask is None:
 		uf_ask = Q_('?Ask (for Subscription):None')
@@ -253,7 +250,7 @@ def get_uf_ask(ask):
 		uf_ask = _('Subscribe')
 	else:
 		uf_ask = ask
-	
+
 	return unicode(uf_ask)
 
 def get_uf_role(role, plural = False):
@@ -323,7 +320,7 @@ def get_uf_chatstate(chatstate):
 	return ''
 
 def is_in_path(name_of_command, return_abs_path = False):
-	# if return_abs_path is True absolute path will be returned 
+	# if return_abs_path is True absolute path will be returned
 	# for name_of_command
 	# on failures False is returned
 	is_in_dir = False
@@ -340,7 +337,7 @@ def is_in_path(name_of_command, return_abs_path = False):
 			if return_abs_path:
 				found_in_which_dir = path_to_directory
 			break
-	
+
 	if found_in_which_dir:
 		abs_path = os.path.join(path_to_directory, name_of_command)
 		return abs_path
@@ -356,10 +353,7 @@ def launch_browser_mailer(kind, uri):
 			pass
 
 	else:
-		if kind == 'url' and not\
-			(uri.startswith('http://') or uri.startswith('https://')):
-			uri = 'http://' + uri
-		elif kind == 'mail' and not uri.startswith('mailto:'):
+		if kind == 'mail' and not uri.startswith('mailto:'):
 			uri = 'mailto:' + uri
 
 		if gajim.config.get('openwith') == 'gnome-open':
@@ -411,12 +405,12 @@ def play_sound(event):
 	if path_to_soundfile == 'beep':
 		print '\a' # make a speaker beep
 		return
-	if not os.path.exists(path_to_soundfile):
+	if path_to_soundfile is None or not os.path.exists(path_to_soundfile):
 		return
 	if os.name == 'nt':
 		try:
 			winsound.PlaySound(path_to_soundfile,
-								winsound.SND_FILENAME|winsound.SND_ASYNC)
+				winsound.SND_FILENAME|winsound.SND_ASYNC)
 		except:
 			pass
 	elif os.name == 'posix':
@@ -442,7 +436,7 @@ def get_file_path_from_dnd_dropped_uri(uri):
 	return path
 
 def from_xs_boolean_to_python_boolean(value):
-	# this is xs:boolean so 'true','false','1','0'
+	# this is xs:boolean so 'true', 'false', '1', '0'
 	# convert those to True/False (python booleans)
 	if value in ('1', 'true'):
 		val = True
@@ -450,10 +444,10 @@ def from_xs_boolean_to_python_boolean(value):
 		val = False
 
 	return val
-			
+
 def ensure_unicode_string(s):
 	# py23 u'abc'.decode('utf-8') raises
-	# python24 does not. is python23 is ooold we can remove this func
+	# python24 does not. if python23 is ooold we can remove this func
 	# FIXME: remove this when we abandon py23
 	if isinstance(s, str):
 		s = s.decode('utf-8')
@@ -464,16 +458,6 @@ def get_xmpp_show(show):
 		return None
 	return show
 
-def one_account_connected():
-	'''returns True if at least one account is connected, else False'''
-	one_connected = False
-	accounts = gajim.connections.keys()
-	for acct in accounts:
-		if gajim.connections[acct].connected > 1:
-			one_connected = True
-			break
-	return one_connected
-
 def get_output_of_command(command):
 	try:
 		child_stdin, child_stdout = os.popen2(command)
@@ -483,7 +467,7 @@ def get_output_of_command(command):
 	output = child_stdout.readlines()
 	child_stdout.close()
 	child_stdin.close()
-	
+
 	return output
 
 def get_global_show():
@@ -497,10 +481,24 @@ def get_global_show():
 			maxi = connected
 	return gajim.SHOW_LIST[maxi]
 
+def get_global_status():
+	maxi = 0
+	for account in gajim.connections:
+		if not gajim.config.get_per('accounts', account,
+			'sync_with_global_status'):
+			continue
+		connected = gajim.connections[account].connected
+		if connected > maxi:
+			maxi = connected
+			status = gajim.connections[account].status
+	return status
+
 def get_icon_name_to_show(contact, account = None):
 	'''Get the icon name to show in online, away, requested, ...'''
 	if account and gajim.awaiting_events[account].has_key(contact.jid):
-		#FIXME: change icon for FT
+		return 'message'
+	if account and gajim.awaiting_events[account].has_key(
+	contact.get_full_jid()):
 		return 'message'
 	if contact.jid.find('@') <= 0: # if not '@' or '@' starts the jid ==> agent
 		return contact.show
@@ -511,7 +509,7 @@ def get_icon_name_to_show(contact, account = None):
 	transport = gajim.get_transport_name_from_jid(contact.jid)
 	if transport:
 		return contact.show
-	return 'not in the roster'
+	return 'not in roster'
 
 def decode_string(string):
 	'''try to decode (to make it Unicode instance) given string'''
@@ -523,5 +521,203 @@ def decode_string(string):
 		except UnicodeError:
 			continue
 		break
-	
+
 	return string
+
+def get_windows_reg_env(varname, default=''):
+	'''asks for paths commonly used but not exposed as ENVs
+	in english Windows 2003 those are:
+	'AppData' = %USERPROFILE%\Application Data (also an ENV)
+	'Desktop' = %USERPROFILE%\Desktop
+	'Favorites' = %USERPROFILE%\Favorites
+	'NetHood' = %USERPROFILE%\NetHood
+	'Personal' = D:\My Documents (PATH TO MY DOCUMENTS)
+	'PrintHood' = %USERPROFILE%\PrintHood
+	'Programs' = %USERPROFILE%\Start Menu\Programs
+	'Recent' = %USERPROFILE%\Recent
+	'SendTo' = %USERPROFILE%\SendTo
+	'Start Menu' = %USERPROFILE%\Start Menu
+	'Startup' = %USERPROFILE%\Start Menu\Programs\Startup
+	'Templates' = %USERPROFILE%\Templates
+	'My Pictures' = D:\My Documents\My Pictures
+	'Local Settings' = %USERPROFILE%\Local Settings
+	'Local AppData' = %USERPROFILE%\Local Settings\Application Data
+	'Cache' = %USERPROFILE%\Local Settings\Temporary Internet Files
+	'Cookies' = %USERPROFILE%\Cookies
+	'History' = %USERPROFILE%\Local Settings\History
+	'''
+
+	if os.name != 'nt':
+		return ''
+
+	val = default
+	try:
+		rkey = win32api.RegOpenKey(win32con.HKEY_CURRENT_USER,
+r'Software\Microsoft\Windows\CurrentVersion\Explorer\User Shell Folders')
+		try:
+			val = str(win32api.RegQueryValueEx(rkey, varname)[0])
+			val = win32api.ExpandEnvironmentStrings(val) # expand using environ
+		except:
+			pass
+	finally:
+		win32api.RegCloseKey(rkey)
+	return val
+ 
+def get_my_pictures_path():
+	'''windows-only atm. [Unix lives in the past]'''
+	return get_windows_reg_env('My Pictures')
+
+def get_desktop_path():
+	if os.name == 'nt':
+		path = get_windows_reg_env('Desktop')
+	else:
+		path = os.path.join(os.path.expanduser('~'), 'Desktop')
+	return path
+
+def get_documents_path():
+	if os.name == 'nt':
+		path = get_windows_reg_env('Personal')
+	else:
+		path = os.path.expanduser('~')
+	return path
+
+# moved from connection.py
+def get_full_jid_from_iq(iq_obj):
+	'''return the full jid (with resource) from an iq as unicode'''
+	return parse_jid(str(iq_obj.getFrom()))
+
+def get_jid_from_iq(iq_obj):
+	'''return the jid (without resource) from an iq as unicode'''
+	jid = get_full_jid_from_iq(iq_obj)
+	return gajim.get_jid_without_resource(jid)
+
+def get_auth_sha(sid, initiator, target):
+	''' return sha of sid + initiator + target used for proxy auth'''
+	return sha.new("%s%s%s" % (sid, initiator, target)).hexdigest()
+
+
+distro_info = {
+	'Arch Linux': '/etc/arch-release',
+	'Aurox Linux': '/etc/aurox-release',
+	'Conectiva Linux': '/etc/conectiva-release',
+	'CRUX': '/usr/bin/crux',
+	'Debian GNU/Linux': '/etc/debian_release',
+	'Debian GNU/Linux': '/etc/debian_version',
+	'Fedora Linux': '/etc/fedora-release',
+	'Gentoo Linux': '/etc/gentoo-release',
+	'Linux from Scratch': '/etc/lfs-release',
+	'Mandrake Linux': '/etc/mandrake-release',
+	'Slackware Linux': '/etc/slackware-release',
+	'Slackware Linux': '/etc/slackware-version',
+	'Solaris/Sparc': '/etc/release',
+	'Source Mage': '/etc/sourcemage_version',
+	'SUSE Linux': '/etc/SuSE-release',
+	'Sun JDS': '/etc/sun-release',
+	'PLD Linux': '/etc/pld-release',
+	'Yellow Dog Linux': '/etc/yellowdog-release',
+	# many distros use the /etc/redhat-release for compatibility
+	# so Redhat is the last
+	'Redhat Linux': '/etc/redhat-release'
+}
+
+def get_random_string_16():
+	''' create random string of length 16'''
+	rng = range(65, 90)
+	rng.extend(range(48, 57))
+	char_sequence = map(lambda e:chr(e), rng)
+	from random import sample
+	return reduce(lambda e1, e2: e1 + e2, 
+			sample(char_sequence, 16))
+	
+def get_os_info():
+	if os.name == 'nt':
+		ver = os.sys.getwindowsversion()
+		ver_format = ver[3], ver[0], ver[1]
+		win_version = {
+			(1, 4, 0): '95',
+			(1, 4, 10): '98',
+			(1, 4, 90): 'ME',
+			(2, 4, 0): 'NT',
+			(2, 5, 0): '2000',
+			(2, 5, 1): 'XP',
+			(2, 5, 2): '2003'
+		}
+		if win_version.has_key(ver_format):
+			return 'Windows' + ' ' + win_version[ver_format]
+		else:
+			return 'Windows'
+	elif os.name == 'posix':
+		executable = 'lsb_release'
+		params = ' --id --codename --release --short'
+		full_path_to_executable = is_in_path(executable, return_abs_path = True)
+		if full_path_to_executable:
+			command = executable + params
+			child_stdin, child_stdout = os.popen2(command)
+			output = temp_failure_retry(child_stdout.readline).strip()
+			child_stdout.close()
+			child_stdin.close()
+			# some distros put n/a in places, so remove those
+			output = output.replace('n/a', '').replace('N/A', '')
+			return output
+
+		# lsb_release executable not available, so parse files
+		for distro_name in distro_info:
+			path_to_file = distro_info[distro_name]
+			if os.path.exists(path_to_file):
+				if os.access(path_to_file, os.X_OK):
+					# the file is executable (f.e. CRUX)
+					# yes, then run it and get the first line of output.
+					text = get_output_of_command(path_to_file)[0]
+				else:
+					fd = open(path_to_file)
+					text = fd.readline().strip() # get only first line
+					fd.close()
+					if path_to_file.endswith('version'):
+						# sourcemage_version has all the info we need
+						if not os.path.basename(path_to_file).startswith('sourcemage'):
+							text = distro_name + ' ' + text
+					elif path_to_file.endswith('aurox-release'):
+						# file doesn't have version
+						text = distro_name
+					elif path_to_file.endswith('lfs-release'): # file just has version
+						text = distro_name + ' ' + text
+				return text
+
+		# our last chance, ask uname and strip it
+		uname_output = get_output_of_command('uname -a | cut -d" " -f1,3')
+		if uname_output is not None:
+			return uname_output[0] # only first line
+	return 'N/A'
+
+def sanitize_filename(filename):
+	'''makes sure the filename we will write does contain only
+	acceptable and latin characters'''
+	filename = punycode_encode(filename) # make it latin chars only
+	filename = filename.replace('/', '_')
+	if os.name == 'nt':
+		filename = filename.replace('?', '').replace(':', '').replace('!', '')\
+			.replace('"', "'")
+	   # 48 is the limit
+		if len(filename) > 48:
+			extension = filename.split('.')[-1]
+			filename = filename[0:48]
+	
+	return filename
+
+def allow_showing_notification(account):
+	'''is it allowed to show nofication?
+	check OUR status and if we allow notifications for that status'''
+	if gajim.config.get('autopopupaway'): # always show notification
+		return True
+	if gajim.connections[account].connected in (2, 3): # we're online or chat
+		return True
+	return False
+
+def allow_popup_window(account):
+	'''is it allowed to popup windows?'''
+	autopopup = gajim.config.get('autopopup')
+	autopopupaway = gajim.config.get('autopopupaway')
+	if autopopup and (autopopupaway or \
+	gajim.connections[account].connected in (2, 3)): # we're online or chat
+		return True
+	return False
