@@ -26,7 +26,6 @@
 
 import gtk
 import gtk.glade
-import gobject
 import pango
 import dialogs
 import gtkgui_helpers
@@ -58,6 +57,11 @@ class GajimThemesWindow:
 		self.italic_togglebutton = self.xml.get_widget('italic_togglebutton')
 		self.themes_tree = self.xml.get_widget('themes_treeview')
 		self.theme_options_vbox = self.xml.get_widget('theme_options_vbox')
+		self.colorbuttons = {}
+		for chatstate in ('active', 'inactive', 'composing', 'paused', 'gone',
+		'muc_msg', 'muc_directed_msg'):
+			self.colorbuttons[chatstate] = self.xml.get_widget(chatstate + \
+				'_colorbutton')
 		model = gtk.ListStore(str)
 		self.themes_tree.set_model(model)
 		col = gtk.TreeViewColumn(_('Theme'))
@@ -74,12 +78,17 @@ class GajimThemesWindow:
 		self.set_theme_options(self.current_theme, self.current_option)
 		
 		self.xml.signal_autoconnect(self)
+		self.window.connect('delete-event', self.on_themese_window_delete_event)
 		self.themes_tree.get_selection().connect('changed', 
 				self.selection_changed)
 		self.window.show_all()
-
+	
+	def on_themese_window_delete_event(self, widget, event):
+		self.window.hide()
+		return True # do NOT destroy the window
+	
 	def on_close_button_clicked(self, widget):
-		self.window.destroy()
+		self.window.hide()
 
 	def on_theme_cell_edited(self, cell, row, new_name):
 		model = self.themes_tree.get_model()
@@ -106,7 +115,6 @@ class GajimThemesWindow:
 			gajim.config.set('roster_theme', new_config_name)
 		model.set_value(iter, 0, new_name)
 		self.current_theme = new_name
-		gajim.interface.instances['preferences'].update_preferences_window()
 
 	def fill_themes_treeview(self):
 		self.xml.get_widget('remove_button').set_sensitive(False)
@@ -148,7 +156,6 @@ class GajimThemesWindow:
 		col = self.themes_tree.get_column(0)
 		path = model.get_path(iter)
 		self.themes_tree.set_cursor(path, col, True)
-		gajim.interface.instances['preferences'].update_preferences_window()
 
 	def on_remove_button_clicked(self, widget):
 		(model, iter) = self.themes_tree.get_selection().get_selected()
@@ -157,12 +164,11 @@ class GajimThemesWindow:
 		if self.current_theme == gajim.config.get('roster_theme'):
 			dialogs.ErrorDialog(
 				_('You cannot delete your current theme'),
-			_('Please first choose another for your current theme.')).get_response()
+			_('Please first choose another for your current theme.'))
 			return
 		self.theme_options_vbox.set_sensitive(False)
 		gajim.config.del_per('themes', self.current_theme)
 		model.remove(iter)
-		gajim.interface.instances['preferences'].update_preferences_window()
 	
 	def set_theme_options(self, theme, option = 'account'):
 		self.no_update = True
@@ -198,6 +204,12 @@ class GajimThemesWindow:
 		self.text_fontbutton.set_sensitive(state)
 		self.no_update = False
 		gajim.interface.roster.change_roster_style(None)
+
+		for chatstate in ('active', 'inactive', 'composing', 'paused', 'gone',
+		'muc_msg', 'muc_directed_msg'):
+			color = gajim.config.get_per('themes', theme, 'state_' + chatstate + \
+				'_color')
+			self.colorbuttons[chatstate].set_color(gtk.gdk.color_parse(color))
 		
 	def on_textcolor_checkbutton_toggled(self, widget):
 		state = widget.get_active()
@@ -247,8 +259,11 @@ class GajimThemesWindow:
 			color_string = gtkgui_helpers.make_color_string(color)
 		else:
 			color_string = ''
+		begin_option = ''
+		if not option.startswith('state'):
+			begin_option = self.current_option
 		gajim.config.set_per('themes', self.current_theme, 
-			self.current_option + option, color_string)
+			begin_option + option, color_string)
 		# use faster functions for this
 		if self.current_option == 'banner':
 			gajim.interface.roster.repaint_themed_widgets()
@@ -324,3 +339,38 @@ class GajimThemesWindow:
 		if font_description.get_style() != pango.STYLE_ITALIC:
 			font_props[1] = True
 		return font_props
+
+	def on_active_colorbutton_color_set(self, widget):
+		self.no_update = True
+		self._set_color(True, widget, 'state_active_color')
+		self.no_update = False
+
+	def on_inactive_colorbutton_color_set(self, widget):
+		self.no_update = True
+		self._set_color(True, widget, 'state_inactive_color')
+		self.no_update = False
+
+	def on_composing_colorbutton_color_set(self, widget):
+		self.no_update = True
+		self._set_color(True, widget, 'state_composing_color')
+		self.no_update = False
+
+	def on_paused_colorbutton_color_set(self, widget):
+		self.no_update = True
+		self._set_color(True, widget, 'state_paused_color')
+		self.no_update = False
+
+	def on_gone_colorbutton_color_set(self, widget):
+		self.no_update = True
+		self._set_color(True, widget, 'state_gone_color')
+		self.no_update = False
+
+	def on_muc_msg_colorbutton_color_set(self, widget):
+		self.no_update = True
+		self._set_color(True, widget, 'state_muc_msg_color')
+		self.no_update = False
+
+	def on_muc_directed_msg_colorbutton_color_set(self, widget):
+		self.no_update = True
+		self._set_color(True, widget, 'state_muc_directed_msg_color')
+		self.no_update = False
