@@ -1,18 +1,11 @@
 ##	gtkgui_helpers.py
 ##
-## Contributors for this file:
-##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Nikos Kouremenos <kourem@gmail.com>
-##	- Dimitur Kirov <dkirov@gmail.com>
-##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
+## Copyright (C) 2004-2005 Vincent Hanquez <tab@snarc.org>
+## Copyright (C) 2005-2006 Nikos Kouremenos <nkour@jabber.org>
+## Copyright (C) 2005 Dimitur Kirov <dkirov@gmail.com>
+## Copyright (C) 2005 Travis Shirk <travis@pobox.com>
+## Copyright (C) 2005 Norman Rasmussen <norman@rasmussen.co.za>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
@@ -52,6 +45,35 @@ from common import helpers
 screen_w = gtk.gdk.screen_width()
 screen_h = gtk.gdk.screen_height()
 
+def popup_emoticons_under_button(menu, button, parent_win):
+	''' pops emoticons menu under button, which is in parent_win'''
+	window_x1, window_y1 = parent_win.get_origin()
+	def position_menu_under_button(menu):
+		# inline function, which will not keep refs, when used as CB
+		button_x, button_y = button.allocation.x, button.allocation.y
+		
+		# now convert them to X11-relative
+		window_x, window_y = window_x1, window_y1
+		x = window_x + button_x
+		y = window_y + button_y
+
+		menu_width, menu_height = menu.size_request()
+
+		## should we pop down or up?
+		if (y + button.allocation.height + menu_height
+			< gtk.gdk.screen_height()):
+			# now move the menu below the button
+			y += button.allocation.height
+		else:
+			# now move the menu above the button
+			y -= menu_height
+
+		# push_in is True so all the menuitems are always inside screen
+		push_in = True
+		return (x, y, push_in)
+
+	menu.popup(None, None, position_menu_under_button, 1, 0)
+	
 def get_theme_font_for_option(theme, option):
 	'''return string description of the font, stored in
 	theme preferences'''
@@ -121,11 +143,9 @@ def get_default_font():
 	
 	return None
 	
-def reduce_chars_newlines(text, max_chars = 0, max_lines = 0, 
-	widget = None):
+def reduce_chars_newlines(text, max_chars = 0, max_lines = 0):
 	'''Cut the chars after 'max_chars' on each line
-	and show only the first 'max_lines'. If there is more text
-	to be shown, display the whole text in tooltip on 'widget'
+	and show only the first 'max_lines'.
 	If any of the params is not present (None or 0) the action
 	on it is not performed'''
 
@@ -145,8 +165,6 @@ def reduce_chars_newlines(text, max_chars = 0, max_lines = 0,
 		reduced_text = reduce(lambda e, e1: e + '\n' + e1, lines)
 	else:
 		reduced_text = ''
-	if reduced_text != text and widget is not None:
-		pass # FIXME show tooltip
 	return reduced_text
 
 def escape_for_pango_markup(string):
@@ -210,20 +228,14 @@ def move_window(window, x, y):
 
 def resize_window(window, w, h):
 	'''resizes window but also checks if huge window or negative values'''
+	if not w or not h:
+		return
 	if w > screen_w:
 		w = screen_w
 	if h > screen_h:
 		h = screen_h
 	window.resize(abs(w), abs(h))
 
-def one_window_opened(typ):
-	for account in gajim.connections:
-		if not gajim.interface.instances[account].has_key(typ):
-			continue
-		if len(gajim.interface.instances[account][typ]):
-			return True
-	return False
-
 class TagInfoHandler(xml.sax.ContentHandler):
 	def __init__(self, tagname1, tagname2):
 		xml.sax.ContentHandler.__init__(self)
@@ -264,7 +276,8 @@ def parse_server_xml(path_to_file):
 def set_unset_urgency_hint(window, unread_messages_no):
 	'''sets/unsets urgency hint in window argument
 	depending if we have unread messages or not'''
-	if gtk.gtk_version >= (2, 8, 0) and gtk.pygtk_version >= (2, 8, 0):
+	if gtk.gtk_version >= (2, 8, 0) and gtk.pygtk_version >= (2, 8, 0) and \
+		gajim.config.get('use_urgency_hint'):
 		if unread_messages_no > 0:
 			window.props.urgency_hint = True
 		else:
@@ -310,8 +323,9 @@ def get_abspath_for_script(scriptname, want_type = False):
 	else:
 		return path_to_script
 
-def get_pixbuf_from_data(file_data):
-	'''Gets image data and returns gtk.gdk.Pixbuf'''
+def get_pixbuf_from_data(file_data, want_type = False):
+	'''Gets image data and returns gtk.gdk.Pixbuf
+	if want_type is True it also returns 'jpeg', 'png' etc'''
 	pixbufloader = gtk.gdk.PixbufLoader()
 	try:
 		pixbufloader.write(file_data)
@@ -320,7 +334,11 @@ def get_pixbuf_from_data(file_data):
 	except gobject.GError: # 'unknown image format'
 		pixbuf = None
 
-	return pixbuf
+	if want_type:
+		typ = pixbufloader.get_format()['name']
+		return pixbuf, typ
+	else:
+		return pixbuf
 
 def get_invisible_cursor():
 	pixmap = gtk.gdk.Pixmap(None, 1, 1, 1)
@@ -411,44 +429,58 @@ def _get_fade_color(treeview, selected, focused):
 			      int(bg.green*p + fg.green*q),
 			      int(bg.blue*p + fg.blue*q))
 
-def get_scaled_pixbuf(pixbuf, type):
-	'''returns scaled pixbuf, keeping ratio etc
-	type is either "chat" or "roster"'''
+def get_scaled_pixbuf(pixbuf, kind):
+	'''returns scaled pixbuf, keeping ratio etc or None
+	kind is either "chat" or "roster" or "notification" or "tooltip"'''
 	
 	# resize to a width / height for the avatar not to have distortion
 	# (keep aspect ratio)
+	width = gajim.config.get(kind + '_avatar_width')
+	height = gajim.config.get(kind + '_avatar_height')
+	if width < 1 or height < 1:
+		return None
 
+	# Pixbuf size
+	pix_width = pixbuf.get_width()
+	pix_height = pixbuf.get_height()
 	# don't make avatars bigger than they are
-	if pixbuf.get_width() < gajim.config.get(type + '_avatar_width') and \
-		pixbuf.get_height() < gajim.config.get(type + '_avatar_height'):
+	if pix_width < width and pix_height < height:
 		return pixbuf # we don't want to make avatar bigger
 
-	ratio = float(pixbuf.get_width()) / float(pixbuf.get_height())
+	ratio = float(pix_width) / float(pix_height)
 	if ratio > 1:
-		w = gajim.config.get(type + '_avatar_width')
+		w = width
 		h = int(w / ratio)
 	else:
-		h = gajim.config.get(type + '_avatar_height')
+		h = height
 		w = int(h * ratio)
 	scaled_buf = pixbuf.scale_simple(w, h, gtk.gdk.INTERP_HYPER)
 	return scaled_buf
 
-def get_avatar_pixbuf_from_cache(jid):
+def get_avatar_pixbuf_from_cache(fjid, is_fake_jid = False):
 	'''checks if jid has cached avatar and if that avatar is valid image
 	(can be shown)
 	returns None if there is no image in vcard
 	returns 'ask' if cached vcard should not be used (user changed his vcard,
 	so we have new sha) or if we don't have the vcard'''
 
+	jid, nick = gajim.get_room_and_nick_from_fjid(fjid)
 	if gajim.config.get('hide_avatar_of_transport') and\
 		gajim.jid_is_transport(jid):
 		# don't show avatar for the transport itself
 		return None
-	
-	if jid not in os.listdir(gajim.VCARDPATH):
+
+	puny_jid = helpers.sanitize_filename(jid)
+	if is_fake_jid:
+		puny_nick = helpers.sanitize_filename(nick)
+		path = os.path.join(gajim.VCARD_PATH, puny_jid, puny_nick)
+	else:
+		path = os.path.join(gajim.VCARD_PATH, puny_jid)
+	if not os.path.isfile(path):
 		return 'ask'
 
-	vcard_dict = gajim.connections.values()[0].get_cached_vcard(jid)
+	vcard_dict = gajim.connections.values()[0].get_cached_vcard(fjid,
+		is_fake_jid)
 	if not vcard_dict: # This can happen if cached vcard is too old
 		return 'ask'
 	if not vcard_dict.has_key('PHOTO'):
@@ -467,5 +499,146 @@ def make_python_month_gtk_month(month):
 
 def make_color_string(color):
 	'''create #aabbcc color string from gtk color'''
-	return '#' + hex(color.red)[-2:] + hex(color.green)[-2:] + \
-		hex(color.blue)[-2:]
+	col = '#'
+	for i in (color.red, color.green, color.blue):
+		h = hex(i)[2:4]
+		if len(h) == 1:
+			h = '0' + h
+		col += h
+	return col
+
+def make_pixbuf_grayscale(pixbuf):
+	pixbuf2 = pixbuf.copy()
+	pixbuf.saturate_and_pixelate(pixbuf2, 0.0, False)
+	return pixbuf2
+
+def get_path_to_generic_or_avatar(generic, jid = None, suffix = None):
+	'''Chooses between avatar image and default image.
+	Returns full path to the avatar image if it exists,
+	otherwise returns full path to the image.'''
+	if jid:
+		puny_jid = helpers.sanitize_filename(jid)
+		path_to_file = os.path.join(gajim.AVATAR_PATH, puny_jid) + suffix
+		if os.path.exists(path_to_file):
+			return path_to_file
+	return os.path.abspath(generic)
+
+def decode_filechooser_file_paths(file_paths):
+	'''decode as UTF-8 under Windows and
+	ask sys.getfilesystemencoding() in POSIX
+	file_paths MUST be LIST'''
+	file_paths_list = list()
+	
+	if os.name == 'nt': # decode as UTF-8 under Windows
+		for file_path in file_paths:
+			file_path = file_path.decode('utf8')
+			file_paths_list.append(file_path)
+	else:
+		for file_path in file_paths:
+			try:
+				file_path = file_path.decode(sys.getfilesystemencoding())
+			except:
+				try:
+					file_path = file_path.decode('utf-8')
+				except:
+					pass
+			file_paths_list.append(file_path)
+	
+	return file_paths_list
+
+def possibly_set_gajim_as_xmpp_handler():
+	'''registers (by default only the first time) xmmp: to Gajim.'''
+	try:
+		import gconf
+		# in try because daemon may not be there
+		client = gconf.client_get_default()
+	except:
+		return
+	
+	
+	path_to_dot_kde = os.path.expanduser('~/.kde')
+	if os.path.exists(path_to_dot_kde):
+		path_to_kde_file = os.path.join(path_to_dot_kde, 
+			'share/services/xmpp.protocol')
+		if not os.path.exists(path_to_kde_file):
+			path_to_kde_file = None
+	else:
+		path_to_kde_file = None
+	
+	if gajim.config.get('set_xmpp://_handler_everytime'):
+			# it's false by default
+			we_set = True
+	elif client.get_string('/desktop/gnome/url-handlers/xmpp/command') is None:
+		# only the first time (GNOME/GCONF)
+		we_set = True
+	elif path_to_kde_file is not None and not os.path.exists(path_to_kde_file):
+		# only the first time (KDE)
+		we_set = True
+	else:
+		we_set = False
+		
+	if not we_set:
+		return
+	
+	path_to_gajim_script, typ = get_abspath_for_script('gajim-remote', True)
+	if path_to_gajim_script:
+		if typ == 'svn':
+			command = path_to_gajim_script + ' open_chat %s'
+		else: # 'installed'
+			command = 'gajim-remote open_chat %s'
+		
+		# setting for GNOME/Gconf
+		client.set_bool('/desktop/gnome/url-handlers/xmpp/enabled', True)
+		client.set_string('/desktop/gnome/url-handlers/xmpp/command', command)
+		client.set_bool('/desktop/gnome/url-handlers/xmpp/needs_terminal', False)
+		
+		# setting for KDE
+		if path_to_kde_file is not None: # user has run kde at least once
+			f = open(path_to_kde_file, 'w')
+			f.write('''\
+[Protocol]
+exec=%s "%%u"
+protocol=xmpp
+input=none
+output=none
+helper=true
+listing=false
+reading=false
+writing=false
+makedir=false
+deleting=false
+icon=gajim
+Description=xmpp
+''' % command)
+			f.close()
+
+def escape_underscore(s):
+	'''Escape underlines to prevent them from being interpreted
+	as keyboard accelerators'''
+	return s.replace('_', '__')
+
+def get_state_image_from_file_path_show(file_path, show):
+	state_file = show.replace(' ', '_')
+	files = []
+	files.append(os.path.join(file_path, state_file + '.png'))
+	files.append(os.path.join(file_path, state_file + '.gif'))
+	image = gtk.Image()
+	image.set_from_pixbuf(None)
+	for file_ in files:
+		if os.path.exists(file_):
+			image.set_from_file(file_)
+			break
+
+	return image
+
+def get_possible_button_event(event):
+	'''mouse or keyboard caused the event?'''
+	if event.type == gtk.gdk.KEY_PRESS:
+		event_button = 0 # no event.button so pass 0
+	else: # BUTTON_PRESS event, so pass event.button
+		event_button = event.button
+
+	return event_button
+
+def destroy_widget(widget):
+	widget.destroy()
