@@ -98,18 +98,6 @@ class SystrayWINAPI:
 						msg_map[key] = value
 			self._message_map.update(msg_map)
 
-	def message_unmap(self, msg, callback=None):
-		if self._message_map.has_key(msg):
-			if callback:
-				cblist = self._message_map[key]
-				if isinstance(cblist, list):
-					if not len(cblist) < 2:
-						for i in xrange(len(cblist)):
-							if cblist[i] == callback:
-								del self._message_map[key][i]
-								return
-			del self._message_map[key]
-
 	def remove_notify_icon(self):
 		""" Removes the notify icon. """
 		if self.notify_icon:
@@ -304,13 +292,16 @@ class SystrayWin32(systray.Systray):
 	def set_img(self):
 		self.tray_ico_imgs = self.load_icos() #FIXME: do not do this here
 		# see gajim.interface.roster.reload_jabber_state_images() to merge
-		self.systray_winapi.remove_notify_icon()
+		
 		if len(self.jids) > 0:
 			state = 'message'
 		else:
 			state = self.status
 		hicon = self.tray_ico_imgs[state]
+		if hicon is None:
+			return
 		
+		self.systray_winapi.remove_notify_icon()
 		self.systray_winapi.add_notify_icon(self.systray_context_menu, hicon,
 			'Gajim')
 		self.systray_winapi.notify_icon.menu = self.systray_context_menu
@@ -336,8 +327,12 @@ class SystrayWin32(systray.Systray):
 			if os.path.exists(path_to_ico):
 				hinst = win32gui.GetModuleHandle(None)
 				img_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
-				image = win32gui.LoadImage(hinst, path_to_ico, win32con.IMAGE_ICON, 
-					0, 0, img_flags)
-				imgs[state] = image
-		
+				try:
+					image = win32gui.LoadImage(hinst, path_to_ico,
+						win32con.IMAGE_ICON, 0, 0, img_flags)
+				except pywintypes.error:
+					imgs[state] = None
+				else:
+					imgs[state] = image
+
 		return imgs
