@@ -1,17 +1,7 @@
-##	vcard.py (has VcardWindow class)
+##	vcard.py (has VcardWindow class and a func get_avatar_pixbuf_encoded_mime)
 ##
-## Contributors for this file:
-##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Nikos Kouremenos <kourem@gmail.com>
-##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
@@ -26,11 +16,12 @@
 import gtk
 import gtk.glade
 import gobject
-import urllib
 import base64
 import mimetypes
 import os
-import sys
+import time
+import locale
+
 import gtkgui_helpers
 import dialogs
 
@@ -61,13 +52,6 @@ def get_avatar_pixbuf_encoded_mime(photo):
 			img_decoded = base64.decodestring(img_encoded)
 		except:
 			pass
-	elif photo.has_key('EXTVAL'):
-		url = photo['EXTVAL']
-		try:
-			fd = urllib.urlopen(url)
-			img_decoded = fd.read()
-		except:
-			pass
 	if img_decoded:
 		pixbuf = gtkgui_helpers.get_pixbuf_from_data(img_decoded)
 	else:
@@ -77,32 +61,44 @@ def get_avatar_pixbuf_encoded_mime(photo):
 class VcardWindow:
 	'''Class for contact's information window'''
 
-	def __init__(self, contact, account, vcard = False):
-		#the contact variable is the jid if vcard is true
+	def __init__(self, contact, account, vcard = False, is_fake = False):
+		# the contact variable is the jid if vcard is true
 		self.xml = gtk.glade.XML(GTKGUI_GLADE, 'vcard_information_window', APP)
 		self.window = self.xml.get_widget('vcard_information_window')
-		self.xml.get_widget('photo_vbuttonbox').set_no_show_all(True)
-		
+
 		self.publish_button = self.xml.get_widget('publish_button')
 		self.retrieve_button = self.xml.get_widget('retrieve_button')
+		self.nickname_entry = self.xml.get_widget('nickname_entry')
+		if not vcard: # Maybe gc_vcard ?
+			self.nickname_entry.set_property('editable', False)
+
 		self.publish_button.set_no_show_all(True)
 		self.retrieve_button.set_no_show_all(True)
-		
-		self.contact = contact #don't use it if vcard is true
+		self.xml.get_widget('photo_vbuttonbox').set_no_show_all(True)
+
+		self.contact = contact # don't use it if vcard is true
 		self.account = account
 		self.vcard = vcard
+		self.is_fake = is_fake
 		self.avatar_mime_type = None
 		self.avatar_encoded = None
 
-		if vcard:
+		if vcard: # we view/edit our own vcard
 			self.jid = contact
-			# remove Jabber tab & show publish/retrieve/set_avatar buttons
+			# remove Jabber tab & show publish/retrieve/close/set_avatar buttons
+			# and make entries and textview editable
 			self.change_to_vcard()
-		else:
-			self.jid = contact.jid
+		else: # we see someone else's vcard
 			self.publish_button.hide()
 			self.retrieve_button.hide()
+			self.jid = contact.jid
 			self.fill_jabber_page()
+			
+			# if we are editing our own vcard publish button should publish
+			# vcard data we have typed including nickname, it's why we connect only
+			# here (when we see someone else's vcard)
+			self.nickname_entry.connect('focus-out-event',
+				self.on_nickname_entry_focus_out_event)
 
 		self.xml.signal_autoconnect(self)
 		self.window.show_all()
@@ -130,21 +126,31 @@ class VcardWindow:
 			gajim.config.set_per('accounts', self.account, 'no_log_for',
 				' '.join(no_log_for))
 	
-	def on_close_button_clicked(self, widget):
-		'''Save contact information and update the roster on the Jabber server'''
-		if self.vcard:
-			self.window.destroy()
-			return
-		#update contact.name if it's not ''
-		name_entry = self.xml.get_widget('nickname_entry')
-		new_name = name_entry.get_text().decode('utf-8')
+	def on_nickname_entry_focus_out_event(self, widget, event):
+		'''Save contact information and update 
+		the roster item on the Jabber server'''
+		new_name = self.nickname_entry.get_text().decode('utf-8')
+		# update contact.name with new nickname if that is not ''
 		if new_name != self.contact.name and new_name != '':
 			self.contact.name = new_name
-			for i in gajim.interface.roster.get_contact_iter(self.contact.jid,
+			# update roster model
+			model = gajim.interface.roster.tree.get_model()
+			for iter_ in gajim.interface.roster.get_contact_iter(self.contact.jid,
 				self.account):
-				gajim.interface.roster.tree.get_model().set_value(i, 1, new_name)
+				model[iter_][1] = new_name
 			gajim.connections[self.account].update_contact(self.contact.jid,
 				self.contact.name, self.contact.groups)
+			# update opened chat window
+			ctrl = gajim.interface.msg_win_mgr.get_control(self.contact.jid,
+				self.account)
+			if ctrl:
+				ctrl.update_ui()
+				win = gajim.interface.msg_win_mgr.get_window(self.contact.jid,
+					self.account)
+				win.redraw_tab(ctrl)
+				win.show_title()
+
+	def on_close_button_clicked(self, widget):		
 		self.window.destroy()
 
 	def on_clear_button_clicked(self, widget):
@@ -152,80 +158,53 @@ class VcardWindow:
 		self.xml.get_widget('PHOTO_image').set_from_pixbuf(None)
 		self.avatar_encoded = None
 
-	def image_is_ok(self, image):
-		if not os.path.exists(image):
-			return False
-		return True
-
-	def update_preview(self, widget):
-		path_to_file = widget.get_preview_filename()
-		if path_to_file is None or os.path.isdir(path_to_file):
-			# nothing to preview or directory
-			# make sure you clean image do show nothing
-			widget.get_preview_widget().set_from_file(None)
-			return
-		try:
-			pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(path_to_file, 100, 100)
-		except gobject.GError:
-			return
-		widget.get_preview_widget().set_from_pixbuf(pixbuf)
-
 	def on_set_avatar_button_clicked(self, widget):
 		f = None
-		dialog = gtk.FileChooserDialog(_('Choose Avatar'), None,
-			gtk.FILE_CHOOSER_ACTION_OPEN,
-			(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-			gtk.STOCK_OPEN, gtk.RESPONSE_OK))
-		dialog.set_default_response(gtk.RESPONSE_OK)
-		filtr = gtk.FileFilter()
-		filtr.set_name(_('All files'))
-		filtr.add_pattern('*')
-		dialog.add_filter(filtr)
-
-		filtr = gtk.FileFilter()
-		filtr.set_name(_('Images'))
-		filtr.add_mime_type('image/png')
-		filtr.add_mime_type('image/jpeg')
-		filtr.add_mime_type('image/gif')
-		filtr.add_mime_type('image/tiff')
-		filtr.add_mime_type('image/x-xpixmap') # xpm
-		dialog.add_filter(filtr)
-		dialog.set_filter(filtr)
-		dialog.set_use_preview_label(False)
-		dialog.set_preview_widget(gtk.Image())
-		dialog.connect('selection-changed', self.update_preview)
-
-		done = False
-		while not done:
-			response = dialog.run()
-			if response == gtk.RESPONSE_OK:
-				f = dialog.get_filename()
+		def on_ok(widget, path_to_file):
+			filesize = os.path.getsize(path_to_file) # in bytes
+			#FIXME: use messages for invalid file for 0.11
+			invalid_file = False
+			msg = ''
+			if os.path.isfile(path_to_file):
+				stat = os.stat(path_to_file)
+				if stat[6] == 0:
+					invalid_file = True
+			else:
+				invalid_file = True
+			if not invalid_file and filesize > 16384: # 16 kb
 				try:
-					f = f.decode(sys.getfilesystemencoding())
-				except UnicodeError:
-					pass
-				else:
-					filesize = os.path.getsize(f) # in bytes
-					if filesize > 32768: # 32 kb
-						dialogs.ErrorDialog(_('The filesize of image "%s" is too large')\
-							% os.path.basename(f),
-						_('The file must not be more than 32 kilobytes.')).get_response()
-						continue
-				if self.image_is_ok(f):
-					done = True
-			else: # Cancel or WM X button
-				done = True
-		dialog.destroy()
+					pixbuf = gtk.gdk.pixbuf_new_from_file(path_to_file)
+					# get the image at 'notification size'
+					# and use that user did not specify in ACE crazy size
+					scaled_pixbuf = gtkgui_helpers.get_scaled_pixbuf(pixbuf,
+						'notification')
+				except gobject.GError, msg: # unknown format
+					# msg should be string, not object instance
+					msg = str(msg)
+					invalid_file = True
+			if invalid_file:
+				if True: # keep identation
+					dialogs.ErrorDialog(_('Could not load image'), msg)
+					return
+			if filesize > 16384:
+					if scaled_pixbuf:
+						path_to_file = os.path.join(gajim.TMP,
+							'avatar_scaled.png')
+						scaled_pixbuf.save(path_to_file, 'png')
+			self.dialog.destroy()
 
-		if response == gtk.RESPONSE_OK:
-			fd = open(f, 'rb')
+			fd = open(path_to_file, 'rb')
 			data = fd.read()
 			pixbuf = gtkgui_helpers.get_pixbuf_from_data(data)
+			# rescale it
+			pixbuf = gtkgui_helpers.get_scaled_pixbuf(pixbuf, 'vcard')
 			image = self.xml.get_widget('PHOTO_image')
 			image.set_from_pixbuf(pixbuf)
 			self.avatar_encoded = base64.encodestring(data)
 			# returns None if unknown type
-			self.avatar_mime_type = mimetypes.guess_type(f)[0]
+			self.avatar_mime_type = mimetypes.guess_type(path_to_file)[0]
+
+		self.dialog = dialogs.ImageChooserDialog(on_response_ok = on_ok)
 
 	def set_value(self, entry_name, value):
 		try:
@@ -238,7 +217,10 @@ class VcardWindow:
 			if i == 'PHOTO':
 				pixbuf, self.avatar_encoded, self.avatar_mime_type = \
 					get_avatar_pixbuf_encoded_mime(vcard[i])
+				if not pixbuf:
+					continue
 				image = self.xml.get_widget('PHOTO_image')
+				pixbuf = gtkgui_helpers.get_scaled_pixbuf(pixbuf, 'vcard')
 				image.set_from_pixbuf(pixbuf)
 				continue
 			if i == 'ADR' or i == 'TEL' or i == 'EMAIL':
@@ -257,7 +239,10 @@ class VcardWindow:
 						vcard[i], 0)
 				else:
 					self.set_value(i + '_entry', vcard[i])
-	
+
+	def set_last_status_time(self):
+		self.fill_status_label()
+
 	def set_os_info(self, resource, client_info, os_info):
 		i = 0
 		client = ''
@@ -281,9 +266,38 @@ class VcardWindow:
 		self.xml.get_widget('client_name_version_label').set_text(client)
 		self.xml.get_widget('os_label').set_text(os)
 
+	def fill_status_label(self):
+		contact_list = gajim.contacts.get_contact(self.account, self.contact.jid)
+		# stats holds show and status message
+		stats = ''
+		one = True # Are we adding the first line ?
+		if contact_list:
+			for c in contact_list:
+				if not one:
+					stats += '\n'
+				stats += helpers.get_uf_show(c.show)
+				if c.status:
+					stats += ': ' + c.status
+				if c.last_status_time:
+					stats += '\n' + _('since %s') % time.strftime('%c',
+						c.last_status_time).decode(locale.getpreferredencoding())
+				one = False
+		elif not self.vcard: # Maybe gc_vcard ?
+			stats = helpers.get_uf_show(self.contact.show)
+			if self.contact.status:
+				stats += ': ' + self.contact.status
+		status_label = self.xml.get_widget('status_label')
+		status_label.set_max_width_chars(15)
+		status_label.set_text(stats)
+
+		tip = gtk.Tooltips()
+		status_label_eventbox = self.xml.get_widget('status_label_eventbox')
+		tip.set_tip(status_label_eventbox, stats)
+
 	def fill_jabber_page(self):
 		tooltips = gtk.Tooltips()
-		self.xml.get_widget('nickname_label').set_text(self.contact.name)
+		self.xml.get_widget('nickname_label').set_text(
+			self.contact.get_shown_name())
 		self.xml.get_widget('jid_label').set_text(self.contact.jid)
 		uf_sub = helpers.get_uf_sub(self.contact.sub)
 		self.xml.get_widget('subscription_label').set_text(uf_sub)
@@ -305,7 +319,7 @@ class VcardWindow:
 		if self.contact.ask == 'subscribe':
 			tooltips.set_tip(eb,
 			_("You are waiting contact's answer about your subscription request"))
-		self.xml.get_widget('nickname_entry').set_text(self.contact.name)
+		self.nickname_entry.set_text(self.contact.name)
 		log = True
 		if self.contact.jid in gajim.config.get_per('accounts', self.account,
 			'no_log_for').split(' '):
@@ -320,45 +334,42 @@ class VcardWindow:
 			+ unicode(self.contact.priority)
 		if not self.contact.status:
 			self.contact.status = ''
-		
-		# stats holds show and status message
-		stats = helpers.get_uf_show(self.contact.show)
-		if self.contact.status:
-			stats += ': ' + self.contact.status
-		gajim.connections[self.account].request_os_info(self.contact.jid,
+
+		# Request list time status
+		gajim.connections[self.account].request_last_status_time(self.contact.jid,
 			self.contact.resource)
+
+		# Request os info in contact is connected
+		if self.contact.show not in ('offline', 'error'):
+			gajim.connections[self.account].request_os_info(self.contact.jid,
+				self.contact.resource)
 		self.os_info = {0: {'resource': self.contact.resource, 'client': '',
 			'os': ''}}
 		i = 1
-		if gajim.contacts[self.account].has_key(self.contact.jid):
-			for c in gajim.contacts[self.account][self.contact.jid]:
+		contact_list = gajim.contacts.get_contact(self.account, self.contact.jid)
+		if contact_list:
+			for c in contact_list:
 				if c.resource != self.contact.resource:
 					resources += '\n%s (%s)' % (c.resource,
 						unicode(c.priority))
-					uf_resources += '\n' + c.resource + _(' resource with priority ')\
-						+ unicode(c.priority)
-					if not c.status:
-						c.status = ''
-					stats += '\n' + c.show + ': ' + c.status
-					gajim.connections[self.account].request_os_info(self.contact.jid,
+					uf_resources += '\n' + c.resource + \
+						_(' resource with priority ') + unicode(c.priority)
+					if c.show not in ('offline', 'error'):
+						gajim.connections[self.account].request_os_info(c.jid,
+							c.resource)
+					gajim.connections[self.account].request_last_status_time(c.jid,
 						c.resource)
 					self.os_info[i] = {'resource': c.resource, 'client': '',
 						'os': ''}
 					i += 1
 		self.xml.get_widget('resource_prio_label').set_text(resources)
-		tip = gtk.Tooltips()
 		resource_prio_label_eventbox = self.xml.get_widget(
 			'resource_prio_label_eventbox')
-		tip.set_tip(resource_prio_label_eventbox, uf_resources)
-		
-		tip = gtk.Tooltips()
-		status_label_eventbox = self.xml.get_widget('status_label_eventbox')
-		tip.set_tip(status_label_eventbox, stats)
-		status_label = self.xml.get_widget('status_label')
-		status_label.set_max_width_chars(15)
-		status_label.set_text(stats)
-		
-		gajim.connections[self.account].request_vcard(self.contact.jid)
+		tooltips.set_tip(resource_prio_label_eventbox, uf_resources)
+
+		self.fill_status_label()
+
+		gajim.connections[self.account].request_vcard(self.contact.jid, self.is_fake)
 
 	def add_to_vcard(self, vcard, entry, txt):
 		'''Add an information to the vCard dictionary'''
@@ -419,7 +430,7 @@ class VcardWindow:
 		if gajim.connections[self.account].connected < 2:
 			dialogs.ErrorDialog(_('You are not connected to the server'),
         		_('Without a connection you can not publish your contact '
-        			'information.')).get_response()
+        		'information.'))
 			return
 		vcard = self.make_vcard()
 		nick = ''
@@ -446,13 +457,12 @@ class VcardWindow:
 			self.xml.get_widget('PHOTO_image').set_from_pixbuf(None)
 			gajim.connections[self.account].request_vcard(self.jid)
 		else:
-			ErrorDialog(_('You are not connected to the server'),
-						_('Without a connection, you can not get your contact information.')).get_response()
+			dialogs.ErrorDialog(_('You are not connected to the server'),
+  		    	_('Without a connection, you can not get your contact information.'))
 
 	def change_to_vcard(self):
 		self.xml.get_widget('information_notebook').remove_page(0)
-		#FIXME: make this string translatable for .10 [thanks Stian]
-		self.xml.get_widget('nickname_label').set_text('Personal details')
+		self.xml.get_widget('nickname_label').set_text(_('Personal details'))
 		
 		self.publish_button.show()
 		self.retrieve_button.show()
